# Generated by Django 3.1.1 on 2021-02-22 15:51

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import django_extensions.db.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("appchance_redlink", "0002_delete_devicenotificationsettings"),
    ]

    operations = [
        migrations.CreateModel(
            name="SentPush",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    django_extensions.db.fields.CreationDateTimeField(
                        auto_now_add=True, verbose_name="created"
                    ),
                ),
                (
                    "modified",
                    django_extensions.db.fields.ModificationDateTimeField(
                        auto_now=True, verbose_name="modified"
                    ),
                ),
                ("content", models.IntegerField()),
                (
                    "user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="sent_notifications",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"get_latest_by": "modified", "abstract": False,},
        ),
    ]



from django.conf import settings
from django.utils import timezone

import pytest
from freezegun import freeze_time

from redlink.models import RedlinkDevice
from redlink.serializers import RedlinkDeviceSerializer
from redlink.tests.factories import RedlinkDeviceFactory


@pytest.mark.django_db
class TestRedlinkDeviceSerializer:
    @freeze_time()
    def test_serialize(self):
        device: RedlinkDevice = RedlinkDeviceFactory()
        serializer = RedlinkDeviceSerializer(device)

        now = timezone.localtime(timezone.now()).strftime(settings.DATETIME_FORMAT)
        assert serializer.data == {
            "id": str(device.id),
            "registration_id": device.registration_id,
            "user": device.user_id,
            "name": device.name,
            "active": device.active,
            "type": device.type,
            "created": now,
            "updated": now,
        }



import factory
from factory import Faker

from redlink.models import RedlinkDevice, SentPush


class RedlinkDeviceFactory(factory.django.DjangoModelFactory):
    name = Faker("pystr", max_chars=10)
    registration_id = Faker("pystr", max_chars=10)
    type = factory.Iterator(["android", "ios"])

    class Meta:
        model = RedlinkDevice


class SentPushFactory(factory.django.DjangoModelFactory):
    content = Faker("pyint")

    class Meta:
        model = SentPush



class BaseNotification:
    TYPE = "base"

    def __init__(self, title, body, **kwargs):
        self.title = title
        self.body = body

    def send_push(self):
        self.redlink_devices.send_push(
            title=self.title, body=self.body, external_data=self.external_data
        )

    @property
    def redlink_devices(self):
        raise NotImplementedError

    @property
    def external_data(self):
        raise NotImplementedError



import uuid

from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models

from django_extensions.db.models import TimeStampedModel

from redlink import app_settings
from redlink.redlink_client import (
    ActionType,
    ReceiverDict,
    ReceiverType,
    RedlinkClient,
)
from redlink.utils import chunk_queryset

User = get_user_model()


class RedlinkClientMixin:
    _redlink_client = None

    @property
    def redlink_client(self) -> RedlinkClient:
        if self._redlink_client is None:
            self._redlink_client = RedlinkClient(settings.REDLINK_API_KEY)
        return self._redlink_client


class RedlinkDeviceQuerySet(RedlinkClientMixin, models.QuerySet):
    def __init__(self, model=None, query=None, using=None, hints=None):
        super().__init__(model=model, query=query, using=using, hints=hints)
        self.CHUNK_SIZE = app_settings.REDLINK_BATCH_SIZE
        self.DEFAULT_LANGUAGE = app_settings.REDLINK_DEFAULT_LANGUAGE

    def user_devices(self, user):
        return self.filter(user=user, active=True)

    def with_marketing_consent(self):
        return self.filter(active=True, settings__marketing=True)

    def with_transactional_consent(self):
        return self.filter(actrive=True, settings__transactional=True)

    def send_push(self, title, body, external_data=None, language=None, **kwargs):
        for registration_ids_chunk in chunk_queryset(
            self.filter(active=True).values("pk", "registration_id"),
            chunk_size=self.CHUNK_SIZE,
        ):
            receivers = [
                ReceiverDict(
                    {
                        "receiver": device["registration_id"],
                        "type": ReceiverType.DEVICE_RECEIVER,
                    }
                )
                for device in registration_ids_chunk
            ]
            if language is None:
                language = self.DEFAULT_LANGUAGE

            if receivers:
                self.redlink_client.send_push(
                    applications=settings.REDLINK_APPLICATIONS,
                    to=receivers,
                    title={language: title},
                    body={language: body},
                    external_data=external_data,
                    action_type=ActionType.WEBVIEW.value,
                    default_language=language,
                    **kwargs
                )


class AbstractRedlinkDevice(RedlinkClientMixin, models.Model):
    DEVICE_TYPES = (
        (u"ios", u"ios"),
        (u"android", u"android"),
    )
    id = models.UUIDField(
        primary_key=True,
        verbose_name="Device ID",
        blank=False,
        unique=True,
        help_text="Unique device identifier",
        max_length=150,
        default=uuid.uuid4,
    )
    registration_id = models.TextField()
    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        related_name="redlink_devices",
    )

    name = models.CharField(max_length=255, verbose_name="Name", null=True)
    active = models.BooleanField(default=True)
    type = models.CharField(choices=DEVICE_TYPES, max_length=10)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    objects = RedlinkDeviceQuerySet.as_manager()

    class Meta:
        abstract = True

    def send_push(
        self, title: str, body: str, external_data=None, language=None, **kwargs
    ):
        if language is None:
            language = self.redlink_client.DEFAULT_LANGUAGE
        self.redlink_client.send_push(
            applications=settings.REDLINK_APPLICATIONS,
            to=[
                ReceiverDict(
                    {
                        "receiver": self.registration_id,
                        "type": ReceiverType.DEVICE_RECEIVER,
                    }
                )
            ],
            title={language: title},
            body={language: body},
            external_data=external_data,
            action_type=ActionType.WEBVIEW.value,
            default_language=language,
            **kwargs
        )


class RedlinkDevice(AbstractRedlinkDevice):
    pass


class SentPush(TimeStampedModel):
    user = models.ForeignKey(
        User, on_delete=models.CASCADE, related_name="sent_notifications", null=True
    )
    content = models.IntegerField()
    is_read = models.BooleanField(default=False)



from django.shortcuts import get_object_or_404
from redlink.models import RedlinkDevice
from redlink.serializers import RedlinkDeviceSerializer, RedlinkCreateDeviceSerializer
from rest_framework import mixins
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import GenericViewSet


class RedlinkDeviceViewSet(
    mixins.CreateModelMixin,
    mixins.RetrieveModelMixin,
    mixins.DestroyModelMixin,
    mixins.ListModelMixin,
    GenericViewSet,
):
    serializer_class = RedlinkDeviceSerializer
    permission_classes = (IsAuthenticated,)
    lookup_field = "registration_id"

    def get_serializer_class(self):
        if self.action == "create":
            return RedlinkCreateDeviceSerializer
        return RedlinkDeviceSerializer

    def get_queryset(self):
        return RedlinkDevice.objects.filter(active=True, user=self.request.user)

    def get_object(self):
        qs = self.get_queryset()
        return get_object_or_404(qs, registration_id=self.kwargs["registration_id"])

    def perform_create(self, serializer):
        device = serializer.save(user=self.request.user)
        self._deactivate_devices_with_equal_registration_id(device=device)
        return device

    def perform_update(self, serializer):
        device = serializer.save(user=self.request.user, session_key="")
        if "registration_id" in serializer.initial_data:
            self._deactivate_devices_with_equal_registration_id(device=device)
        return device

    def _deactivate_devices_with_equal_registration_id(self, device):
        RedlinkDevice.objects.filter(registration_id=device.registration_id,).exclude(
            id=device.id
        ).update(active=False)


# pylint: disable=protected-access
from unittest.mock import ANY, MagicMock, patch

from django.conf import settings

import pytest

from redlink.errors import (
    RedlinkAuthorizationError,
    RedlinkNotFoundError,
    RedlinkServerError,
)
from redlink.redlink_client import (
    ActionType,
    ReceiverType,
    RedlinkClient,
)


class TestRedlinkClient:
    def test_parse__minimal_data(self,):
        redlink_client = RedlinkClient("test_api_key")
        payload = redlink_client._parse_payload(
            applications=["XXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"],
            title="Test title",
            body="Test body",
            to=[{"receiver": "test-device", "type": ReceiverType.DEVICE_RECEIVER}],
        )
        assert payload == {
            "applications": ["XXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"],
            "to": [
                {"receiver": "test-device", "type": ReceiverType.DEVICE_RECEIVER.value}
            ],
            "title": {"en": "Test title"},
            "body": {"en": "Test body"},
            "defaultLanguage": "en",
            "scheduleTime": ANY,
            "externalData": {},
            "action": {"type": ActionType.NONE.value, "url": ""},
        }

    def test_parse__full_data(self):
        redlink_client = RedlinkClient("test_api_key")
        payload = redlink_client._parse_payload(
            applications=["XXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"],
            title={"pl": "Test title"},
            body={"pl": "Test body"},
            to=[
                {
                    "receiver": "example-device-id",
                    "type": ReceiverType.DEVICE_RECEIVER,
                    "external_id": "ext_id_1",
                },
                {
                    "receiver": "+48123123123",
                    "type": ReceiverType.NUMBER_RECEIVER,
                    "external_id": "ext_id_2",
                },
                {
                    "receiver": "exail@example.com",
                    "type": ReceiverType.EMAIL_RECEIVER,
                    "external_id": "ext_id_3",
                },
            ],
            default_language="pl",
            silent=False,
            sound="/relative-sound-file-path",
            image="http://absolute.url/to/image",
            schedule_time="2020-07-09 12:00:00",
            ttl=123412,
            external_data={"test": "x", "test1": False},
            subtitle="subtitle-only-ios",
            lockscreen_visibility=1,
            icon_small="/relative-icon-file-path",
            icon_large="/relative-icon-file-path",
            action_url="http://absolute.url/to/redirect/on/click",
            action_type=2,
            action_buttons=[
                {
                    "button": 1,
                    "icon": "icon-path",
                    "action": {"url": "string", "type": 0},
                }
            ],
        )

        assert payload == {
            "applications": ["XXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"],
            "to": [
                {
                    "receiver": "example-device-id",
                    "type": ReceiverType.DEVICE_RECEIVER.value,
                    "external_id": "ext_id_1",
                },
                {
                    "receiver": "+48123123123",
                    "type": ReceiverType.NUMBER_RECEIVER.value,
                    "external_id": "ext_id_2",
                },
                {
                    "receiver": "exail@example.com",
                    "type": ReceiverType.EMAIL_RECEIVER.value,
                    "external_id": "ext_id_3",
                },
            ],
            "title": {"pl": "Test title"},
            "body": {"pl": "Test body"},
            "defaultLanguage": "pl",
            "image": "http://absolute.url/to/image",
            "silent": False,
            "sound": "/relative-sound-file-path",
            "scheduleTime": "2020-07-09 12:00:00",
            "ttl": 123412,
            "externalData": {"test": "x", "test1": False},
            "advanced": {
                "subtitle": "subtitle-only-ios",
                "lockscreenVisibility": 1,
                "icon": {
                    "small": "/relative-icon-file-path",
                    "large": "/relative-icon-file-path",
                },
            },
            "action": {"url": "http://absolute.url/to/redirect/on/click", "type": 2},
            "actionButtons": [
                {
                    "button": 1,
                    "icon": "icon-path",
                    "action": {"url": "string", "type": 0},
                }
            ],
        }

    @patch("redlink.redlink_client.requests")
    def test_send_push(self, requests_mock):
        expected_response = MagicMock(status_code=200)
        requests_mock.post.return_value = expected_response
        redlink_client = RedlinkClient("test_api_key")
        response = redlink_client.send_push(
            applications=["XXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"],
            title="Test title",
            body="Test body",
            to=[{"receiver": "test-device", "type": ReceiverType.DEVICE_RECEIVER}],
        )
        assert response == expected_response

    @patch("redlink.redlink_client.requests")
    def test_send_push__anauthorized(self, requests_mock):
        requests_mock.post.return_value = MagicMock(status_code=401)
        redlink_client = RedlinkClient("test_api_key")
        with pytest.raises(RedlinkAuthorizationError):
            redlink_client.send_push(
                applications=["XXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"],
                title="Test title",
                body="Test body",
                to=[{"receiver": "test-device", "type": ReceiverType.DEVICE_RECEIVER}],
            )

    @patch("redlink.redlink_client.requests")
    def test_send_push__not_found(self, requests_mock):
        requests_mock.post.return_value = MagicMock(status_code=404)
        redlink_client = RedlinkClient("test_api_key")
        with pytest.raises(RedlinkNotFoundError):
            redlink_client.send_push(
                applications=["XXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX"],
                title="Test title",
                body="Test body",
                to=[{"receiver": "test-device", "type": ReceiverType.DEVICE_RECEIVER}],
            )

    @patch("redlink.redlink_client.requests")
    def test_send_push__server_error(self, requests_mock):
        requests_mock.post.return_value = MagicMock(status_code=500)
        redlink_client = RedlinkClient(settings.REDLINK_API_KEY)
        with pytest.raises(RedlinkServerError):
            redlink_client.send_push(
                applications=["testapp"],
                title="Test title",
                body="Test body",
                to=[{"receiver": "test-device", "type": ReceiverType.DEVICE_RECEIVER}],
            )

    @pytest.mark.skip("Requires external redlink request.")
    def test_send_push__external(self):
        redlink_client = RedlinkClient(settings.REDLINK_API_KEY)
        response = redlink_client.send_push(
            applications=["e334cc4d-fc0d-4842-9b37-a6175d58a98f"],
            title="Test title",
            body="Test body",
            to=[
                {
                    "receiver": "cwwnofzRKWY:APA91bFnWFnFjG_kKUmNoNIRGt2SqHCGhVLp0hvz3fSQR-3xLuJFNe-TJ6R2ShFrUFjmaEwMWk"
                    "FQpWXXoE4vs-qXVG-HNAK_T1ac0RYhf7u1U3wBC20V2J5R8JKupmG24_1QUgSDYfZ4",
                    "type": ReceiverType.DEVICE_RECEIVER,
                }
            ],
        )
        assert response.status_code == 200



# Generated by Django 3.1.1 on 2021-02-01 13:00

import uuid

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models

import redlink.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="RedlinkDevice",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        help_text="Unique device identifier",
                        primary_key=True,
                        serialize=False,
                        unique=True,
                        verbose_name="Device ID",
                    ),
                ),
                ("registration_id", models.TextField()),
                (
                    "name",
                    models.CharField(max_length=255, null=True, verbose_name="Name"),
                ),
                ("active", models.BooleanField(default=True)),
                (
                    "type",
                    models.CharField(
                        choices=[("ios", "ios"), ("android", "android")], max_length=10
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True)),
                ("updated", models.DateTimeField(auto_now=True)),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="redlink_devices",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={"abstract": False,},
            bases=(redlink.models.RedlinkClientMixin, models.Model),
        ),
        migrations.CreateModel(
            name="DeviceNotificationSettings",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("marketing", models.BooleanField(default=True)),
                ("transactional", models.BooleanField(default=True)),
                (
                    "device",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="settings",
                        to="appchance_redlink.redlinkdevice",
                    ),
                ),
            ],
            options={"abstract": False,},
        ),
    ]



from rest_framework import serializers
from rest_framework.serializers import ModelSerializer

from redlink.models import RedlinkDevice


class RedlinkDeviceSerializer(ModelSerializer):
    created = serializers.DateTimeField(required=False)

    class Meta:
        model = RedlinkDevice
        fields = "__all__"
        extra_kwargs = {"id": {"read_only": True, "required": False}}

    def validate(self, attrs):
        self.Meta.model.objects.filter()
        return attrs


class RedlinkCreateDeviceSerializer(ModelSerializer):
    class Meta:
        model = RedlinkDevice
        fields = (
            "registration_id",
            "name",
            "active",
            "type",
        )



from django.contrib import admin

from redlink.models import RedlinkDevice, SentPush


class RedlinkDeviceAdminAbstract:
    list_display = (
        "id",
        "registration_id",
        "user",
        "name",
        "type",
        "created",
        "updated",
    )
    readonly_fields = ("user",)
    search_fields = ("id", "registration_id", "user__id")


@admin.register(RedlinkDevice)
class RedlinkDeviceAdmin(RedlinkDeviceAdminAbstract, admin.ModelAdmin):
    pass

@admin.register(SentPush)
class SentPushAdmin(admin.ModelAdmin):
    pass



# Generated by Django 3.1.1 on 2021-02-02 13:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("appchance_redlink", "0001_initial"),
    ]

    operations = [
        migrations.DeleteModel(name="DeviceNotificationSettings",),
    ]
